@@ -5,7 +5,7 @@ from typing import Optional, Dict, Any, Tuple, cast
 
 import pymediainfo
 from fffw.encoding import codecs
-from fffw.graph.meta import VideoMeta, AudioMeta, AUDIO, VIDEO, from_media_info
+from fffw.graph.meta import VideoMeta, AudioMeta, AUDIO, VIDEO, from_media_info, TS
 from fffw.encoding.inputs import input_file, Stream
 from fffw.encoding.filters import Scale
 from fffw.encoding.outputs import output_file
@@ -21,8 +21,14 @@ SCALE = 'scale'
 # Used metadata keys
 AUDIO_DURATION = 'audio_duration'
 AUDIO_SAMPLING_RATE = 'audio_sampling_rate'
+SAMPLES_COUNT = 'samples'
 VIDEO_DURATION = 'video_duration'
 VIDEO_FRAME_RATE = 'video_frame_rate'
+FRAMES_COUNT = 'frames'
+DAR = 'aspect'
+PAR = 'par'
+WIDTH = 'width'
+HEIGHT = 'height'
 
 # Handy media info log format
 MEDIA_INFO_MSG_FORMAT = """%s media info:
@@ -195,10 +201,120 @@ class Transcoder(LoggerMixin):
         # Validate ffmpeg result
         self.validate(source_media_info, dest_media_info)
 
-    @staticmethod
-    def get_meta_data(filename: str) -> Tuple[Optional[AudioMeta],
-                                              Optional[VideoMeta]]:
+    def fix_par(self, t: pymediainfo.Track) -> None:
+        """
+        Fix PAR to satisfy equation DAR = width / height * PAR.
+        """
+        dar = float(t.__dict__.get('display_aspect_ratio', 0))
+        par = float(t.__dict__.get('pixel_aspect_ratio', 0))
+        width = int(t.__dict__.get('width', 0))
+        height = int(t.__dict__.get('height', 0))
+
+        if not (width and height):
+            # not enough info to fix PAR
+            self.logger.debug("width or height unknown, can't restore PAR metadata")
+            return
+        ratio = width / height
+
+        if not par and dar:
+            self.logger.debug("restoring par from dar")
+            par = dar / ratio
+        elif not dar and par:
+            self.logger.debug("restoring dar from par")
+            dar = par * ratio
+        elif not dar and not par:
+            self.logger.debug("setting aspects to defaults")
+            par = 1.0
+            dar = ratio
+
+        # at this moment we know all 4 variables, checking equation
+        if abs(dar - ratio * par) >= 0.001:  # see fffw.meta.VideoMeta.validate
+            # par is least reliable value, using it to fix equation
+            par = dar / ratio
+        t.__dict__['display_aspect_ratio'] = f'{dar:.3f}'
+        t.__dict__['pixel_aspect_ratio'] = f'{par:.3f}'
+
+    def fix_frames(self, t: pymediainfo.Track) -> None:
+        """
+        Fix frames count to satisfy equation:
+
+        Duration = FPS * frames
+        """
+        duration = float(TS(t.__dict__.get('duration', 0)))  # duration in seconds
+        frame_rate = float(t.__dict__.get('frame_rate', 0))
+        frames = int(t.__dict__.get('frame_count', 0))
+
+        if not duration and frames and frame_rate:
+            self.logger.debug("restoring video duration")
+            duration = frames / frame_rate
+        elif not frames and duration and frame_rate:
+            self.logger.debug("restoring frames")
+            frames = round(duration * frame_rate)
+        elif not frame_rate and duration and frames:
+            self.logger.debug("restoging frame_rate")
+            frame_rate = frames / duration
+        elif not all([frames, frame_rate, duration]):
+            # 2 of 3 variables are unknown, or even all of them.
+            # can't restore metadata
+            return
+
+        # checking equation
+        if abs(frames - duration * frame_rate) > 1:
+            # frames is least reliable value
+            frame_rate = frames / duration
+
+        t.__dict__['frame_rate'] = f'{frame_rate:.3f}'
+        t.__dict__['duration'] = f'{duration:.3f}'
+        t.__dict__['frame_count'] = f'{frames}'
+
+    def fix_samples(self, t: pymediainfo.Track) -> None:
+        """
+        Fix sample count to satisfy equation:
+
+        Duration = Sampling rate * samples
+        """
+        duration = float(TS(t.__dict__.get('duration', 0)))  # duration in seconds
+        sampling_rate = float(t.__dict__.get('sampling_rate', 0))
+        samples = int(t.__dict__.get('samples_count', 0))
+
+        if not duration and samples and sampling_rate:
+            self.logger.debug("restoring audio duration")
+            duration = samples / sampling_rate
+        elif not samples and duration and sampling_rate:
+            self.logger.debug("restoring samples")
+            samples = round(duration * sampling_rate)
+        elif not sampling_rate and duration and samples:
+            self.logger.debug("restoging sampling_rate")
+            sampling_rate = samples / duration
+        elif not all([samples, sampling_rate, duration]):
+            # 2 of 3 variables are unknown, or even all of them.
+            # can't restore metadata
+            return
+
+        # fix sampling rate type
+        sampling_rate = round(sampling_rate)
+        # handle duration rounding
+        duration = round(duration, 3)
+
+        # checking equation
+        if abs(samples - duration * sampling_rate) > 1:
+            # samples is least reliable data, because sampling rate has common values like 48000,
+            # and duration is more reliable.
+            samples = round(duration * sampling_rate)
+
+        t.__dict__['sampling_rate'] = f'{sampling_rate}'
+        t.__dict__['duration'] = f'{duration:.3f}'
+        t.__dict__['samples_count'] = f'{samples}'
+
+    def get_meta_data(self, filename: str
+                      ) -> Tuple[Optional[AudioMeta], Optional[VideoMeta]]:
         result: pymediainfo.MediaInfo = pymediainfo.MediaInfo.parse(filename)
+        for t in result.tracks:
+            if t.track_type in ('Video', 'Image'):
+                self.fix_par(t)
+                self.fix_frames(t)
+            elif t.track_type == 'Audio':
+                self.fix_samples(t)
         metadata = from_media_info(result)
         video_meta = None
         audio_meta = None
